@@ -1,11 +1,17 @@
 import json
+from typing import Dict
 
 import requests
 
 from smile_id_core.Signature import Signature
 from smile_id_core.ServerError import ServerError
 
-__all__ = ["Utilities"]
+__all__ = ["Utilities", "get_signature", "validate_sec_params", "sid_server_map"]
+
+sid_server_map = {
+    0: "https://testapi.smileidentity.com/v1",
+    1: "https://api.smileidentity.com/v1",
+}
 
 
 class Utilities:
@@ -16,20 +22,17 @@ class Utilities:
         self.api_key = api_key
         self.sid_server = sid_server
         if sid_server in [0, 1]:
-            sid_server_map = {
-                0: "https://3eydmgh10d.execute-api.us-west-2.amazonaws.com/test",
-                1: "https://la7am6gdm8.execute-api.us-west-2.amazonaws.com/prod",
-            }
             self.url = sid_server_map[sid_server]
         else:
             self.url = sid_server
 
-    def get_job_status(self, partner_params, option_params, sec_key, timestamp):
-        if sec_key is None:
-            sec_key_object = self.__get_sec_key()
-            sec_key = sec_key_object["sec_key"]
-            timestamp = sec_key_object["timestamp"]
+    def get_job_status(self, partner_params, option_params, sec_params):
+        if sec_params is None:
+            sec_params = get_signature(
+                self.partner_id, self.api_key, option_params.get("signature")
+            )
 
+        validate_sec_params(sec_params)
         Utilities.validate_partner_params(partner_params)
         if not option_params or option_params is None:
             options = {
@@ -43,16 +46,13 @@ class Utilities:
             partner_params.get("user_id"),
             partner_params.get("job_id"),
             options,
-            sec_key,
-            timestamp,
+            sec_params,
         )
 
-    def __query_job_status(self, user_id, job_id, option_params, sec_key, timestamp):
+    def __query_job_status(self, user_id, job_id, option_params, sec_params):
         job_status = Utilities.execute_post(
             self.url + "/job_status",
-            self.__configure_job_query(
-                user_id, job_id, option_params, sec_key, timestamp
-            ),
+            self.__configure_job_query(user_id, job_id, option_params, sec_params),
         )
         if job_status.status_code != 200:
             raise ServerError(
@@ -68,17 +68,19 @@ class Utilities:
             timestamp = job_status_json_resp["timestamp"]
             server_signature = job_status_json_resp["signature"]
             signature = Signature(self.partner_id, self.api_key)
-            valid = signature.confirm_sec_key(timestamp, server_signature)
+            if option_params.get("signature"):
+                valid = signature.confirm_signature(timestamp, server_signature)
+            else:
+                valid = signature.confirm_sec_key(timestamp, server_signature)
             if not valid:
                 raise ServerError(
                     "Unable to confirm validity of the job_status response"
                 )
             return job_status
 
-    def __configure_job_query(self, user_id, job_id, options, sec_key, timestamp):
+    def __configure_job_query(self, user_id, job_id, options, sec_params):
         return {
-            "sec_key": sec_key,
-            "timestamp": timestamp,
+            **sec_params,
             "partner_id": self.partner_id,
             "job_id": job_id,
             "user_id": user_id,
@@ -86,39 +88,32 @@ class Utilities:
             "history": options["return_history"],
         }
 
-    def __get_sec_key(self):
-        sec_key_gen = Signature(self.partner_id, self.api_key)
-        return sec_key_gen.generate_sec_key()
-
     @staticmethod
     def validate_partner_params(partner_params):
         if not partner_params:
             raise ValueError("Please ensure that you send through partner params")
 
         if (
-            not partner_params["user_id"]
-            or not partner_params["job_id"]
-            or not partner_params["job_type"]
+            not partner_params.get("user_id")
+            or not partner_params.get("job_id")
+            or not partner_params.get("job_type")
         ):
             raise ValueError("Partner Parameter Arguments may not be null or empty")
 
-        if not isinstance(partner_params["user_id"], str):
+        if not isinstance(partner_params.get("user_id"), str):
             raise ValueError("Please ensure user_id is a string")
 
-        if not isinstance(partner_params["job_id"], str):
+        if not isinstance(partner_params.get("job_id"), str):
             raise ValueError("Please ensure job_id is a string")
 
-        if not isinstance(partner_params["job_id"], str):
-            raise ValueError("Please ensure job_id is a string")
-
-        if not isinstance(partner_params["job_type"], int):
+        if not isinstance(partner_params.get("job_type"), int):
             raise ValueError("Please ensure job_id is a number")
 
     @staticmethod
     def validate_id_params(
         sid_server, id_info_params, partner_params, use_validation_api=True
     ):
-        if not id_info_params["entered"]:
+        if not id_info_params.get("entered"):
             return
 
         for field in ["country", "id_type", "id_number"]:
@@ -158,10 +153,6 @@ class Utilities:
     @staticmethod
     def get_smile_id_services(sid_server):
         if sid_server in [0, 1]:
-            sid_server_map = {
-                0: "https://3eydmgh10d.execute-api.us-west-2.amazonaws.com/test",
-                1: "https://la7am6gdm8.execute-api.us-west-2.amazonaws.com/prod",
-            }
             url = sid_server_map[sid_server]
         else:
             url = sid_server
@@ -198,3 +189,27 @@ class Utilities:
             },
         )
         return resp
+
+
+def validate_sec_params(sec_key_dict: Dict):
+    if not sec_key_dict.get("sec_key") and not sec_key_dict.get("signature"):
+        raise Exception("Missing key, must provide a 'sec_key' or 'signature' field")
+    if not sec_key_dict.get("timestamp"):
+        raise Exception("Missing 'timestamp' field")
+
+
+def get_signature(partner_id, api_key, is_signature):
+    sec_key_gen = Signature(partner_id, api_key)
+    sec_key_object = (
+        sec_key_gen.generate_signature()
+        if is_signature
+        else sec_key_gen.generate_sec_key()
+    )
+    sec_key = sec_key_object.get("sec_key")
+    signature = sec_key_object.get("signature")
+    payload = {"timestamp": sec_key_object["timestamp"]}
+    if sec_key:
+        payload.update({"sec_key": sec_key})
+    else:
+        payload.update({"signature": signature})
+    return payload
