@@ -1,12 +1,18 @@
 import json
 import time
+from typing import Dict
 
 import requests
 
 from smile_id_core.image_upload import generate_zip_file, validate_images
 from smile_id_core.IdApi import IdApi
 from smile_id_core.Signature import Signature
-from smile_id_core.Utilities import Utilities
+from smile_id_core.Utilities import (
+    Utilities,
+    get_signature,
+    validate_sec_params,
+    sid_server_map,
+)
 from smile_id_core.ServerError import ServerError
 
 __all__ = ["WebApi"]
@@ -23,10 +29,6 @@ class WebApi:
         self.utilities = None
 
         if sid_server in [0, 1]:
-            sid_server_map = {
-                0: "https://3eydmgh10d.execute-api.us-west-2.amazonaws.com/test",
-                1: "https://la7am6gdm8.execute-api.us-west-2.amazonaws.com/prod",
-            }
             self.url = sid_server_map[sid_server]
         else:
             self.url = sid_server
@@ -41,7 +43,7 @@ class WebApi:
     ):
 
         Utilities.validate_partner_params(partner_params)
-        job_type = partner_params["job_type"]
+        job_type = partner_params.get("job_type")
 
         if not id_info_params:
             if job_type == 5:
@@ -60,11 +62,6 @@ class WebApi:
                 "entered": False,
             }
 
-        if job_type == 5:
-            return self.__call_id_api(
-                partner_params, id_info_params, use_validation_api
-            )
-
         if not options_params:
             options_params = {
                 "return_job_status": True,
@@ -72,6 +69,11 @@ class WebApi:
                 "return_images": False,
             }
 
+        if job_type == 5:
+            return self.__call_id_api(
+                partner_params, id_info_params, use_validation_api, options_params
+            )
+
         self.__validate_options(options_params)
         validate_images(images_params)
         Utilities.validate_id_params(
@@ -79,13 +81,11 @@ class WebApi:
         )
         self.__validate_return_data(options_params)
 
-        sec_key_object = self.__get_sec_key()
-        sec_key = sec_key_object["sec_key"]
-        timestamp = sec_key_object["timestamp"]
+        sec_params = self._get_security_key_params(options_params)
 
         prep_upload = WebApi.execute_http(
             self.url + "/upload",
-            self.__prepare_prep_upload_payload(partner_params, sec_key, timestamp),
+            self.__prepare_prep_upload_payload(partner_params, sec_params),
         )
         if prep_upload.status_code != 200:
             raise ServerError(
@@ -99,19 +99,18 @@ class WebApi:
             smile_job_id = prep_upload_json_resp["smile_job_id"]
             zip_stream = generate_zip_file(
                 partner_id=self.partner_id,
-                sec_key=sec_key,
-                timestamp=timestamp,
                 callback_url=self.call_back_url,
                 image_params=images_params,
                 partner_params=partner_params,
                 id_info_params=id_info_params,
                 upload_url=upload_url,
+                sec_params=sec_params,
             )
             upload_response = WebApi.upload(upload_url, zip_stream)
             if upload_response.status_code != 200:
                 raise ServerError(
                     "Failed to post entity to {}, status={}, response={}".format(
-                        upload_url, prep_upload.status_code, prep_upload.json()
+                        upload_url, upload_response.status_code, upload_response.json()
                     )
                 )
 
@@ -120,22 +119,28 @@ class WebApi:
                     self.partner_id, self.api_key, self.sid_server
                 )
                 job_status = self.poll_job_status(
-                    0,
-                    partner_params,
-                    options_params,
-                    sec_key_object["sec_key"],
-                    sec_key_object["timestamp"],
+                    0, partner_params, options_params, sec_params
                 )
-                job_status_response = job_status.json()
-                job_status_response["success"] = True
-                job_status_response["smile_job_id"] = smile_job_id
                 return job_status
             else:
                 return {"success": True, "smile_job_id": smile_job_id}
 
-    def __call_id_api(self, partner_params, id_info_params, use_validation_api):
+    def _get_security_key_params(self, options_params):
+        return get_signature(
+            self.partner_id, self.api_key, options_params.get("signature")
+        )
+
+    def __call_id_api(
+        self,
+        partner_params: Dict,
+        id_info_params: Dict,
+        use_validation_api: bool,
+        options_params: Dict,
+    ):
         id_api = IdApi(self.partner_id, self.api_key, self.sid_server)
-        return id_api.submit_job(partner_params, id_info_params, use_validation_api)
+        return id_api.submit_job(
+            partner_params, id_info_params, use_validation_api, options_params
+        )
 
     def __validate_options(self, options_params):
         if not self.call_back_url and not options_params:
@@ -158,25 +163,25 @@ class WebApi:
         sec_key_gen = Signature(self.partner_id, self.api_key)
         return sec_key_gen.generate_sec_key()
 
-    def __prepare_prep_upload_payload(self, partner_params, sec_key, timestamp):
+    def __prepare_prep_upload_payload(self, partner_params: Dict, sec_params: Dict):
+        validate_sec_params(sec_params)
+
         return {
             "file_name": "selfie.zip",
-            "timestamp": timestamp,
-            "sec_key": sec_key,
             "smile_client_id": self.partner_id,
             "partner_params": partner_params,
             "model_parameters": {},
             "callback_url": self.call_back_url,
+            **sec_params,
         }
 
     def poll_job_status(
-        self, counter, partner_params, options_params, sec_key=None, timestamp=None
+        self, counter, partner_params, options_params, sec_params: Dict
     ):
-        if sec_key is None:
-            sec_key_object = self.__get_sec_key()
-            sec_key = sec_key_object["sec_key"]
-            timestamp = sec_key_object["timestamp"]
+        if sec_params is None:
+            sec_params = self._get_security_key_params(options_params)
 
+        validate_sec_params(sec_params)
         counter = counter + 1
         if counter < 4:
             time.sleep(2)
@@ -184,12 +189,12 @@ class WebApi:
             time.sleep(4)
 
         job_status = self.utilities.get_job_status(
-            partner_params, options_params, sec_key, timestamp
+            partner_params, options_params, sec_params
         )
         job_status_response = job_status.json()
         if not job_status_response["job_complete"] and counter < 20:
-            self.poll_job_status(
-                counter, partner_params, options_params, sec_key, timestamp
+            return self.poll_job_status(
+                counter, partner_params, options_params, sec_params
             )
 
         return job_status
