@@ -2,6 +2,7 @@ import json
 import zipfile
 import io
 import os
+from smile_id_core.Utilities import validate_sec_params
 
 
 class ApiVersion:
@@ -20,8 +21,7 @@ def generate_zip_file(
     partner_params,
     image_params,
     id_info_params,
-    sec_key,
-    timestamp,
+    sec_params,
 ):
     info_json = prepare_info_json(
         partner_id,
@@ -30,15 +30,16 @@ def generate_zip_file(
         partner_params,
         image_params,
         id_info_params,
-        sec_key,
-        timestamp,
+        sec_params,
     )
     zip_buffer = io.BytesIO()
     with zipfile.ZipFile(zip_buffer, "a", zipfile.ZIP_DEFLATED, False) as zip_file:
         zip_file.writestr("info.json", data=json.dumps(info_json))
         for image in image_params:
             image_file_path = image["image"]
-            if image_file_path.lower().endswith(IMAGE_FILE_EXTENSIONS):
+            if isinstance(image_file_path, str) and image_file_path.lower().endswith(
+                IMAGE_FILE_EXTENSIONS
+            ):
                 # TODO: do we really silently skip a file if its extension is different?
                 zip_file.write(image_file_path, os.path.basename(image_file_path))
     return zip_buffer.getvalue()
@@ -51,9 +52,9 @@ def prepare_info_json(
     partner_params,
     image_params,
     id_info_params,
-    sec_key,
-    timestamp,
+    sec_params,
 ):
+    validate_sec_params(sec_params)
     return {
         "package_information": {
             "apiVersion": {
@@ -64,10 +65,9 @@ def prepare_info_json(
             "language": "python",
         },
         "misc_information": {
-            "sec_key": sec_key,
+            **sec_params,
             "retry": "false",
             "partner_params": partner_params,
-            "timestamp": timestamp,
             "file_name": "selfie.zip",
             "smile_client_id": partner_id,
             "callback_url": callback_url,
@@ -95,7 +95,9 @@ def prepare_image_payload(image_params):
 
 
 def prepare_image_entry_dict(image, image_type_id, **_):
-    if image.lower().endswith(IMAGE_FILE_EXTENSIONS):
+    if isinstance(image, bytes):
+        image = image.decode("utf-8")
+    elif image.lower().endswith(IMAGE_FILE_EXTENSIONS):
         return {
             "image_type_id": image_type_id,
             "image": "",
@@ -116,7 +118,9 @@ def validate_images(images_params):
         raise ValueError("Please ensure that you send through image details as a list")
 
     for image in images_params:
-        if image["image"].lower().endswith(IMAGE_FILE_EXTENSIONS):
+        if isinstance(image["image"], str) and image["image"].lower().endswith(
+            IMAGE_FILE_EXTENSIONS
+        ):
             if not os.path.exists(image["image"]):
                 raise FileNotFoundError(
                     "No such file or directory %s" % (image["image"])
